package de.unikoblenz.fgbks.core.dmn.domain.vdmn.impl;

import de.unikoblenz.fgbks.core.dmn.domain.ids.DecisionTableId;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDecision;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDecisionTable;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnInputColumn;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnOutputColumn;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnRule;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import org.apache.commons.lang3.Validate;
import org.camunda.bpm.model.dmn.HitPolicy;
import org.camunda.bpm.model.dmn.instance.DecisionTable;

/**
 * Implementation of {@link VDmnDecisionTable}
 */
public class VDmnDecisionTableImpl extends AbstractVDmnElement implements VDmnDecisionTable {

  protected DecisionTable decisionTable;
  protected VDmnDecision dmnDecision;
  protected HitPolicy hitPolicy;
  protected List<VDmnOutputColumn> dmnOutputColumns;
  protected List<VDmnInputColumn> dmnInputColumns;
  protected List<VDmnRule> dmnRules;

  private VDmnDecisionTableImpl() {
    super();
    dmnOutputColumns = new ArrayList<>();
    dmnInputColumns = new ArrayList<>();
    dmnRules = new ArrayList<>();
  }

  @Override
  public DecisionTable getDecisionTable() {
    return decisionTable;
  }

  @Override
  public VDmnDecision getVDmnDecision() {
    return dmnDecision;
  }

  @Override
  public HitPolicy getHitPolicy() {
    return hitPolicy;
  }

  @Override
  public List<VDmnOutputColumn> getVDmnOutputColumns() {
    return Collections.unmodifiableList(dmnOutputColumns);
  }

  @Override
  public List<VDmnInputColumn> getVDmnInputColumns() {
    return Collections.unmodifiableList(dmnInputColumns);
  }

  @Override
  public List<VDmnRule> getVDmnRules() {
    return Collections.unmodifiableList(dmnRules);
  }

  /**
   * Get a new instance of a builder.
   *
   * @return a new builder instance
   */
  public static Builder getBuilder() {
    return new VDmnDecisionTableImpl().new Builder();
  }

  public class Builder extends AbstractDmnElementBuilder<VDmnDecisionTableImpl> {

    public Builder withDecisionTable(DecisionTable decisionTable) {
      value.decisionTable = decisionTable;
      return this;
    }

    public Builder withDecisionTableId(DecisionTableId decisionTableId) {
      value.id = decisionTableId;
      return this;
    }

    public Builder withVDmnDecision(VDmnDecision dmnDecision) {
      value.dmnDecision = dmnDecision;
      return this;
    }

    public Builder withHitPolicy(HitPolicy hitPolicy) {
      value.hitPolicy = hitPolicy;
      return this;
    }

    public Builder addVDmnOutputColumn(VDmnOutputColumn outputColumn) {
      value.dmnOutputColumns.add(Validate.notNull(outputColumn));
      return this;
    }

    public Builder addVDmnOutputColumnFromBuilder(
        VDmnOutputColumnImpl.Builder outputColumnBuilder) {
      addVDmnOutputColumn(Validate.notNull(outputColumnBuilder.getUnbuildValue()));
      outputColumnBuilder.withDecisionTable(value);
      return this;
    }

    public Builder addVDmnInputColumn(VDmnInputColumn inputColumn) {
      value.dmnInputColumns.add(Validate.notNull(inputColumn));
      return this;
    }

    public Builder addDmnInputColumnFromBuilder(VDmnInputColumnImpl.Builder inputColumnBuilder) {
      addVDmnInputColumn(Validate.notNull(inputColumnBuilder.getUnbuildValue()));
      inputColumnBuilder.withVDmnDecisionTable(value);
      return this;
    }

    public Builder addVDmnRule(VDmnRule dmnRule) {
      value.dmnRules.add(Validate.notNull(dmnRule));
      return this;
    }

    public Builder addVDmnRuleFromBuilder(VDmnRuleImpl.Builder dmnRuleBuilder) {
      addVDmnRule(Validate.notNull(dmnRuleBuilder.getUnbuildValue()));
      dmnRuleBuilder.withVDmnDecisionTable(value);
      return this;
    }

    @Override
    protected void validate() {
      super.validate();
      Validate.notNull(value.id);
      Validate.notNull(value.dmnDecision);
      Validate.notNull(value.hitPolicy);
    }
  }
}
