package de.unikoblenz.fgbks.core.dmn.domain.vdmn.impl;

import de.unikoblenz.fgbks.base.domain.Description;
import de.unikoblenz.fgbks.base.domain.Text;
import de.unikoblenz.fgbks.core.dmn.domain.ids.OutputEntryId;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnOutputColumn;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnOutputValue;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnRule;
import org.apache.commons.lang3.Validate;
import org.camunda.bpm.model.dmn.instance.OutputEntry;

/**
 * Implementation of {@link VDmnOutputValue}.
 */
public class VDmnOutputValueImpl extends AbstractVDmnValue implements VDmnOutputValue {

  protected OutputEntry outputEntry;

  private VDmnOutputValueImpl() {
    super();
  }

  @Override
  public OutputEntry getOutputEntry() {
    return outputEntry;
  }

  /**
   * Get a new instance of a builder.
   *
   * @return a new builder instance
   */
  public static Builder getBuilder() {
    return new VDmnOutputValueImpl().new Builder();
  }

  public class Builder extends AbstractDmnElementBuilder<VDmnOutputValueImpl> {

    public Builder withOutputEntry(OutputEntry outputEntry) {
      value.outputEntry = outputEntry;
      return this;
    }

    public Builder withOutputEntryId(OutputEntryId outputEntryId) {
      value.id = outputEntryId;
      return this;
    }

    public Builder withVDmnRule(VDmnRule dmnRule) {
      value.rule = dmnRule;
      return this;
    }

    public Builder withText(String text) {
      value.text = new Text(text);
      return this;
    }

    public Builder withDescription(String description) {
      value.description = new Description(description);
      return this;
    }

    public Builder withVDmnOutputColumn(VDmnOutputColumn dmnColumn) {
      value.column = dmnColumn;
      return this;
    }

    @Override
    protected void validate() {
      super.validate();
      Validate.notNull(value.id);
      Validate.notNull(value.rule);
      Validate.notNull(value.text);
      Validate.notNull(value.column);
    }
  }
}
