package de.unikoblenz.fgbks.core.dmn.domain.vdmn.impl;

import de.unikoblenz.fgbks.core.dmn.domain.common.RowNumber;
import de.unikoblenz.fgbks.core.dmn.domain.ids.RuleId;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnDecisionTable;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnInputValue;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnOutputValue;
import de.unikoblenz.fgbks.core.dmn.domain.vdmn.VDmnRule;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import org.apache.commons.lang3.Validate;
import org.camunda.bpm.model.dmn.instance.Rule;

/**
 * Implementation of {@link VDmnRule}.
 */
public class VDmnRuleImpl extends AbstractVDmnElement implements VDmnRule {

  protected Rule rule;
  protected RowNumber rowNumber;
  protected VDmnDecisionTable dmnDecisionTable;
  protected List<VDmnInputValue> inputValues;
  protected List<VDmnOutputValue> outputValues;

  protected VDmnRuleImpl() {
    inputValues = new ArrayList<>(10);
    outputValues = new ArrayList<>(3);
  }

  @Override
  public Rule getRule() {
    return rule;
  }

  @Override
  public RowNumber getRowNumber() {
    return rowNumber;
  }

  @Override
  public VDmnDecisionTable getVDmnDecisionTable() {
    return dmnDecisionTable;
  }

  @Override
  public List<VDmnInputValue> getVDmnInputValues() {
    return Collections.unmodifiableList(inputValues);
  }

  @Override
  public List<VDmnOutputValue> getVDmnOutputValues() {
    return Collections.unmodifiableList(outputValues);
  }

  /**
   * Get a new instance of a builder.
   *
   * @return a new builder instance
   */
  public static Builder getBuilder() {
    return new VDmnRuleImpl().new Builder();
  }

  public class Builder extends AbstractDmnElementBuilder<VDmnRuleImpl> {

    public Builder withRule(Rule rule) {
      value.rule = rule;
      return this;
    }

    public Builder withRowNumber(RowNumber rowNumber) {
      value.rowNumber = rowNumber;
      return this;
    }

    public Builder withRuleId(RuleId ruleId) {
      value.id = ruleId;
      return this;
    }

    public Builder withVDmnDecisionTable(VDmnDecisionTable vDmnDecisionTable) {
      value.dmnDecisionTable = vDmnDecisionTable;
      return this;
    }

    public Builder addVDmnInputValue(VDmnInputValue inputValue) {
      value.inputValues.add(Validate.notNull(inputValue));
      return this;
    }

    public Builder addVDmnInputValueFromBuilder(VDmnInputValueImpl.Builder inputValueBuilder) {
      addVDmnInputValue(Validate.notNull(inputValueBuilder.getUnbuildValue()));
      inputValueBuilder.withVDmnRule(value);
      return this;
    }

    public Builder addVDmnOutputValue(VDmnOutputValue outputValue) {
      value.outputValues.add(Validate.notNull(outputValue));
      return this;
    }

    public Builder addVDmnOutputValueFromBuilder(VDmnOutputValueImpl.Builder outputValueBuilder) {
      addVDmnOutputValue(Validate.notNull(outputValueBuilder.getUnbuildValue()));
      outputValueBuilder.withVDmnRule(value);
      return this;
    }

    @Override
    protected void validate() {
      super.validate();
      Validate.notNull(value.id);
      Validate.notNull(value.rowNumber);
      Validate.notNull(value.dmnDecisionTable);
    }
  }
}
